==> teliver-1.0.3.tgz: package/android/src/main/java/com/teliver/TeliverModule.java <==
package com.teliver;

import android.location.Location;
import android.util.Log;

import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.Callback;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.WritableMap;
import com.facebook.react.modules.core.DeviceEventManagerModule;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.annotations.SerializedName;

import io.teliver.sdk.core.InitStatusListener;
import io.teliver.sdk.core.TLog;
import io.teliver.sdk.core.Teliver;
import io.teliver.sdk.core.TripListener;
import io.teliver.sdk.models.MarkerOption;
import io.teliver.sdk.models.PushData;
import io.teliver.sdk.models.TrackingBuilder;
import io.teliver.sdk.models.Trip;
import io.teliver.sdk.models.User;
import io.teliver.sdk.util.TUtils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

@SuppressWarnings("WeakerAccess")
public class TeliverModule extends ReactContextBaseJavaModule {

    /**
     * Base Context
     */
    private ReactContext context;

    /**
     * Constructor
     *
     * @param reactContext = base context of library
     */
    public TeliverModule(ReactApplicationContext reactContext) {
        super(reactContext);
        this.context = reactContext;
    }

    /**
     * @return this class name
     */
    @Override
    public String getName() {
        return "Teliver";
    }

    /**
     * @param apiKey   = Unique API KEY retrieved from teliver app panel
     * @param callback = callback to java script
     */
    @ReactMethod
    public void initSDK(String apiKey, final Callback callback) {
        try {
            Teliver.init(context, apiKey);
            Teliver.setInitStatusListener(new InitStatusListener() {
                @Override
                public void onSuccess() {
                    if (callback != null)
                        callback.invoke(true, "Init Success");
                }

                @Override
                public void OnFailure(String reason) {
                    if (callback != null)
                        callback.invoke(false, "Init Failure::" + reason);
                }
            });
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * @param userObj = Save User obj and for receive push
     */
    @ReactMethod
    public void identifyUser(String userObj) {
        try {
            if (TUtils.clearNull(userObj).isEmpty())
                showLogMessage("User Object should not be empty or null !");
            else
                Teliver.identifyUser(getGson().fromJson(userObj, User.class));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Unregister user object
     */
    @ReactMethod
    public void unregisterUser() {
        Teliver.unregisterUser();
    }

    /**
     * @param trackingObject = Start Tracking with custom options like title,
     *                       message, image .
     */
     /*
    @ReactMethod
    public void startTrackingWithOption(String trackingObject) {
        try {
            if (TUtils.clearNull(trackingObject).isEmpty())
                showLogMessage("Tracking Object should not be empty or null !");
            else {
                TrackingObject mainObj = getGson().fromJson(trackingObject, TrackingObject.class);
                List<TrackingData> dataList = mainObj.getTrackingObjects();
                List<MarkerOption> options = new ArrayList<>();
                for (TrackingData data : dataList) {
                    MarkerOption option = new MarkerOption(data.getTrackingId());
                    option.setMarkerTitle(data.getTitle());
                    option.setMarkerSnippet(data.getSnippet());
                    option.setIconMarker(getResourceFile(data.getIconResource(),
                            mainObj.getPackageName()));
                    options.add(option);
                }
                TrackingBuilder builder = new TrackingBuilder(options);
                builder.withTitle(mainObj.getPageTitle());
                Teliver.startTracking(builder.build());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    private int getResourceFile(String fName, String packageName) {
        try {
            return context.getResources().getIdentifier(fName, "drawable", packageName);
        } catch (Exception e) {
            return R.drawable.marker_red;
        }
    }
    */

    /**
     * @param trackingId = Start Tracking with Single Tracking ID
     */
    @ReactMethod
    public void startTracking(String trackingId) {
        try {
            if (TUtils.clearNull(trackingId).isEmpty())
                showLogMessage("Tracking ID should not be empty or null !");
            else {
                List<MarkerOption> options = new ArrayList<>();
                options.add(new MarkerOption(trackingId));
                Teliver.startTracking(new TrackingBuilder(options)
                        .withTitle("#" + trackingId).build());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * @param trackingIds = Start Tracking with Comma Separated IDs
     */
    @ReactMethod
    public void startMultiTracking(String trackingIds) {
        try {
            if (TUtils.clearNull(trackingIds).isEmpty())
                showLogMessage("Tracking IDs should not be empty or null !");
            else {
                List<MarkerOption> options = new ArrayList<>();
                for (String trackingId : trackingIds.split(","))
                    options.add(new MarkerOption(trackingId));
                Teliver.startTracking(new TrackingBuilder(options).build());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * @param trackingId = Stop Tracking
     */
    @ReactMethod
    public void stopTracking(String trackingId) {
        stopTracking(Collections.singletonList(trackingId));
    }

    /**
     * @param trackingIds =  Stop tracking with Comma Separated IDS
     */
    @ReactMethod
    public void stopMultiTracking(String trackingIds) {
        try {
            if (TUtils.clearNull(trackingIds).isEmpty())
                showLogMessage("Tracking IDs should not be empty or null !");
            else
                stopTracking(Arrays.asList(trackingIds.split(",")));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Local Method
     *
     * @param allIds = Tracking IDS
     */
    private void stopTracking(List<String> allIds) {
        Teliver.stopTracking(allIds);
    }

    /**
     * Trip Listener or Callback initialization
     */
    @ReactMethod
    public void setTripListener() {
        Teliver.setTripListener(new MTripListener());
    }

    /**
     * @param trackingId = Tracking ID
     * @param pushObj    = Push message in json-object format
     * @param tag        = message or data
     */
    @ReactMethod
    public void sendEventPush(String trackingId, String pushObj, String tag) {
        try {
            if (TUtils.clearNull(pushObj).isEmpty())
                showLogMessage("Push Data should not be empty or null !");
            else {
                sendEventPush(trackingId, pushObj);
                tagLocation(trackingId, tag);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * @param trackingId = Tracking ID
     * @param pushObj    = Push message in json-object format
     */
    @ReactMethod
    public void sendEventPush(String trackingId, String pushObj) {
        try {
            if (TUtils.clearNull(pushObj).isEmpty())
                showLogMessage("Push Data should not be empty or null !");
            else
                Teliver.sendEventPush(trackingId, getGson().fromJson(
                        pushObj, PushData.class));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @ReactMethod
    public void startTrip(String tripId) {
        try {
            Teliver.startTrip(tripId);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @ReactMethod
    public void stopTrip(String tripId) {
        try {
            Teliver.stopTrip(tripId);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * @param trackingId = Tracking ID
     * @param tag        = message or data
     */
    @ReactMethod
    public void tagLocation(String trackingId, String tag) {
        Teliver.tagLocation(trackingId, tag);
    }

    /**
     * @param visibility = teliver log visibility
     */
    @ReactMethod
    public void setLogVisibility(boolean visibility) {
        TLog.setVisible(visibility);
    }

    public Gson getGson() {
        return new GsonBuilder().create();
    }

    private void showLogMessage(String message) {
        Log.v("Teliver Module::", "::" + message);
    }

    class MTripListener implements TripListener {

        @Override
        public void onTripStarted(Trip tripDetails) {
            sendEvent("onTripStarted", getGson().toJson(tripDetails));
        }

        @Override
        public void onLocationUpdate(Location location) {
            sendEvent("onLocationUpdate", getGson().toJson(location));
        }

        @Override
        public void onTripEnded(String trackingID) {
            sendEvent("onTripEnded", trackingID);
        }

        @Override
        public void onTripError(String reason) {
            sendEvent("onTripError", reason);
        }

        /**
         * Sending Event to Java Script Class
         *
         * @param eventName =name
         * @param data      = data
         */
        private void sendEvent(String eventName, String data) {
            try {
                WritableMap eventData = Arguments.createMap();
                eventData.putString("data", data);
                context.getJSModule(DeviceEventManagerModule.RCTDeviceEventEmitter.class)
                        .emit(eventName, eventData);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    static class TrackingData {

        @SerializedName("tracking_id")
        private String trackingId;

        private String title, snippet;

        @SerializedName("image_drawable")
        private String iconResource;

        String getTrackingId() {
            return trackingId;
        }

        String getTitle() {
            return title;
        }

        String getSnippet() {
            return snippet;
        }

        String getIconResource() {
            return iconResource;
        }
    }

    static class TrackingObject {

        @SerializedName("page_title")
        private String pageTitle;

        @SerializedName("package_name")
        private String packageName;

        @SerializedName("tracking_data")
        private List<TrackingData> trackingData;

        public String getPackageName() {
            return packageName;
        }

        String getPageTitle() {
            return pageTitle;
        }

        List<TrackingData> getTrackingObjects() {
            return trackingData == null ? new ArrayList<TrackingData>() : trackingData;
        }
    }
}

==> teliver-1.0.4.tgz: package/android/src/main/java/com/teliver/TeliverModule.java <==
package com.teliver;

import android.location.Location;
import android.util.Log;

import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.Callback;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.WritableMap;
import com.facebook.react.modules.core.DeviceEventManagerModule;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.annotations.SerializedName;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import io.teliver.sdk.core.InitStatusListener;
import io.teliver.sdk.core.TLog;
import io.teliver.sdk.core.Teliver;
import io.teliver.sdk.core.TripListener;
import io.teliver.sdk.models.MarkerOption;
import io.teliver.sdk.models.PushData;
import io.teliver.sdk.models.TrackingBuilder;
import io.teliver.sdk.models.Trip;
import io.teliver.sdk.models.User;
import io.teliver.sdk.util.TUtils;

@SuppressWarnings("WeakerAccess")
public class TeliverModule extends ReactContextBaseJavaModule {

    /**
     * Base Context
     */
    private ReactContext context;

    /**
     * Constructor
     *
     * @param reactContext = base context of library
     */
    public TeliverModule(ReactApplicationContext reactContext) {
        super(reactContext);
        this.context = reactContext;
    }

    /**
     * @return this class name
     */
    @Override
    public String getName() {
        return "Teliver";
    }

    /**
     * @param apiKey   = Unique API KEY retrieved from teliver app panel
     * @param callback = callback to java script
     */
    @ReactMethod
    public void initSDK(String apiKey, final Callback callback) {
        try {
            Teliver.init(context, apiKey);
            Teliver.setInitStatusListener(new InitStatusListener() {
                @Override
                public void onSuccess() {
                    if (callback != null)
                        callback.invoke(true, "Init Success");
                }

                @Override
                public void OnFailure(String reason) {
                    if (callback != null)
                        callback.invoke(false, "Init Failure::" + reason);
                }
            });
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * @param userObj = Save User obj and for receive push
     */
    @ReactMethod
    public void identifyUser(String userObj) {
        try {
            if (TUtils.clearNull(userObj).isEmpty())
                showLogMessage("User Object should not be empty or null !");
            else
                Teliver.identifyUser(getGson().fromJson(userObj, User.class));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Unregister user object
     */
    @ReactMethod
    public void unregisterUser() {
        Teliver.unregisterUser();
    }

    /**
     * @param trackingObject = Start Tracking with custom options like title,
     *                       message, image .
     */
     /*
    @ReactMethod
    public void startTrackingWithOption(String trackingObject) {
        try {
            if (TUtils.clearNull(trackingObject).isEmpty())
                showLogMessage("Tracking Object should not be empty or null !");
            else {
                TrackingObject mainObj = getGson().fromJson(trackingObject, TrackingObject.class);
                List<TrackingData> dataList = mainObj.getTrackingObjects();
                List<MarkerOption> options = new ArrayList<>();
                for (TrackingData data : dataList) {
                    MarkerOption option = new MarkerOption(data.getTrackingId());
                    option.setMarkerTitle(data.getTitle());
                    option.setMarkerSnippet(data.getSnippet());
                    option.setIconMarker(getResourceFile(data.getIconResource(),
                            mainObj.getPackageName()));
                    options.add(option);
                }
                TrackingBuilder builder = new TrackingBuilder(options);
                builder.withTitle(mainObj.getPageTitle());
                Teliver.startTracking(builder.build());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    private int getResourceFile(String fName, String packageName) {
        try {
            return context.getResources().getIdentifier(fName, "drawable", packageName);
        } catch (Exception e) {
            return R.drawable.marker_red;
        }
    }
    */

    /**
     * @param trackingId = Start Tracking with Single Tracking ID
     */
    @ReactMethod
    public void startTracking(String trackingId) {
        try {
            if (TUtils.clearNull(trackingId).isEmpty())
                showLogMessage("Tracking ID should not be empty or null !");
            else {
                List<MarkerOption> options = new ArrayList<>();
                options.add(new MarkerOption(trackingId));
                Teliver.startTracking(new TrackingBuilder(options)
                        .withTitle("#" + trackingId).build());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * @param trackingIds = Start Tracking with Comma Separated IDs
     */
    @ReactMethod
    public void startMultiTracking(String trackingIds) {
        try {
            if (TUtils.clearNull(trackingIds).isEmpty())
                showLogMessage("Tracking IDs should not be empty or null !");
            else {
                List<MarkerOption> options = new ArrayList<>();
                for (String trackingId : trackingIds.split(","))
                    options.add(new MarkerOption(trackingId));
                Teliver.startTracking(new TrackingBuilder(options).build());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * @param trackingId = Stop Tracking
     */
    @ReactMethod
    public void stopTracking(String trackingId) {
        stopTracking(Collections.singletonList(trackingId));
    }

    /**
     * @param trackingIds =  Stop tracking with Comma Separated IDS
     */
    @ReactMethod
    public void stopMultiTracking(String trackingIds) {
        try {
            if (TUtils.clearNull(trackingIds).isEmpty())
                showLogMessage("Tracking IDs should not be empty or null !");
            else
                stopTracking(Arrays.asList(trackingIds.split(",")));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Local Method
     *
     * @param allIds = Tracking IDS
     */
    private void stopTracking(List<String> allIds) {
        Teliver.stopTracking(allIds);
    }

    /**
     * Trip Listener or Callback initialization
     */
    @ReactMethod
    public void setTripListener() {
        Teliver.setTripListener(new MTripListener());
    }

    /**
     * @param trackingId = Tracking ID
     * @param pushObj    = Push message in json-object format
     * @param tag        = message or data
     */
    @ReactMethod
    public void sendEventPush(String trackingId, String pushObj, String tag) {
        try {
            if (TUtils.clearNull(pushObj).isEmpty())
                showLogMessage("Push Data should not be empty or null !");
            else {
                sendEventPush(trackingId, pushObj);
                tagLocation(trackingId, tag);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * @param trackingId = Tracking ID
     * @param pushObj    = Push message in json-object format
     */
    @ReactMethod
    public void sendEventPush(String trackingId, String pushObj) {
        try {
            if (TUtils.clearNull(pushObj).isEmpty())
                showLogMessage("Push Data should not be empty or null !");
            else
                Teliver.sendEventPush(trackingId, getGson().fromJson(
                        pushObj, PushData.class));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @ReactMethod
    public void startTrip(String tripId, Callback callback) {
        try {
            Teliver.startTrip(tripId);
            updateTripStatus(callback);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void updateTripStatus(final Callback callback) {
        try {
            Teliver.setTripListener(new TripListener() {
                @Override
                public void onTripStarted(Trip tripDetails) {
                    if (callback != null)
                        callback.invoke("onTripStarted", getGson().toJson(tripDetails));
                }

                @Override
                public void onLocationUpdate(Location location) {

                }

                @Override
                public void onTripEnded(String trackingID) {
                    if (callback != null)
                        callback.invoke("onTripEnded", trackingID);
                }

                @Override
                public void onTripError(String reason) {
                    if (callback != null)
                        callback.invoke("onTripError", reason);
                }
            });
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @ReactMethod
    public void stopTrip(String tripId, Callback callback) {
        try {
            Teliver.stopTrip(tripId);
            updateTripStatus(callback);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * @param trackingId = Tracking ID
     * @param tag        = message or data
     */
    @ReactMethod
    public void tagLocation(String trackingId, String tag) {
        Teliver.tagLocation(trackingId, tag);
    }

    /**
     * @param visibility = teliver log visibility
     */
    @ReactMethod
    public void setLogVisibility(boolean visibility) {
        TLog.setVisible(visibility);
    }

    public Gson getGson() {
        return new GsonBuilder().create();
    }

    private void showLogMessage(String message) {
        Log.v("Teliver Module::", "::" + message);
    }

    class MTripListener implements TripListener {

        @Override
        public void onTripStarted(Trip tripDetails) {
            sendEvent("onTripStarted", getGson().toJson(tripDetails));
        }

        @Override
        public void onLocationUpdate(Location location) {
            sendEvent("onLocationUpdate", getGson().toJson(location));
        }

        @Override
        public void onTripEnded(String trackingID) {
            sendEvent("onTripEnded", trackingID);
        }

        @Override
        public void onTripError(String reason) {
            sendEvent("onTripError", reason);
        }

        /**
         * Sending Event to Java Script Class
         *
         * @param eventName =name
         * @param data      = data
         */
        private void sendEvent(String eventName, String data) {
            try {
                WritableMap eventData = Arguments.createMap();
                eventData.putString("data", data);
                context.getJSModule(DeviceEventManagerModule.RCTDeviceEventEmitter.class)
                        .emit(eventName, eventData);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    static class TrackingData {

        @SerializedName("tracking_id")
        private String trackingId;

        private String title, snippet;

        @SerializedName("image_drawable")
        private String iconResource;

        String getTrackingId() {
            return trackingId;
        }

        String getTitle() {
            return title;
        }

        String getSnippet() {
            return snippet;
        }

        String getIconResource() {
            return iconResource;
        }
    }

    static class TrackingObject {

        @SerializedName("page_title")
        private String pageTitle;

        @SerializedName("package_name")
        private String packageName;

        @SerializedName("tracking_data")
        private List<TrackingData> trackingData;

        public String getPackageName() {
            return packageName;
        }

        String getPageTitle() {
            return pageTitle;
        }

        List<TrackingData> getTrackingObjects() {
            return trackingData == null ? new ArrayList<TrackingData>() : trackingData;
        }
    }
}
